@@ -22,6 +22,17 @@
 
 QHash<quint64, pki_base*> db_base::lookup;
 
+void db_base::restart_timer()
+{
+	killTimer(secondsTimer);
+	killTimer(minutesTimer);
+	killTimer(hoursTimer);
+
+	secondsTimer = startTimer(1000);
+	minutesTimer = startTimer(MSECS_PER_MINUTE);
+	hoursTimer = startTimer(MSECS_PER_HOUR);
+}
+
 db_base::db_base(MainWindow *mw)
 	:QAbstractItemModel(NULL)
 {
@@ -31,6 +42,8 @@ db_base::db_base(MainWindow *mw)
 	colResizing = 0;
 	currentIdx = QModelIndex();
 	class_name = "base";
+	secondsTimer = minutesTimer = hoursTimer = 0;
+	restart_timer();
 }
 
 db_base::~db_base()
@@ -124,6 +137,7 @@ void db_base::loadContainer()
 	} else {
 		allHeaders.fromData(view);
 	}
+	restart_timer();
 	emit columnsContentChanged();
 }
 
@@ -141,10 +155,10 @@ dbheaderList db_base::getHeaders()
 	dbheaderList h;
 	/* "No." handled in XcaProxyModel */
 	h << new dbheader(HD_internal_name, true, tr("Internal name"))
-	  << new dbheader(HD_counter, false, tr("No."))
-	  << new dbheader(HD_primary_key, false, tr("Primary key"),
+	  << new num_dbheader(HD_counter, false, tr("No."))
+	  << new num_dbheader(HD_primary_key, false, tr("Primary key"),
 			tr("Database unique number"))
-	  << new dbheader(HD_creation, false, tr("Date"),
+	  << new date_dbheader(HD_creation, false, tr("Date"),
 			tr("Date of creation or insertion"))
 	  << new dbheader(HD_source, false, tr("Source"),
 			tr("Generated, Imported, Transformed"))
@@ -221,6 +235,7 @@ void db_base::insertPKI(pki_base *pki)
 	lookup[pki->getSqlItemId().toULongLong()] = pki;
 	inToCont(pki);
 	TransCommit();
+	restart_timer();
 	emit columnsContentChanged();
 }
 
@@ -435,8 +450,9 @@ QVariant db_base::data(const QModelIndex &index, int role) const
 	switch (role) {
 		case Qt::EditRole:
 		case Qt::DisplayRole:
-			if (hd->id == HD_internal_name || item->isVisible() == 1)
+			if (hd->id==HD_internal_name || item->isVisible()==1)
 				return item->column_data(hd);
+			break;
 		case Qt::DecorationRole:
 			return item->getIcon(hd);
 		case Qt::TextAlignmentRole:
@@ -447,6 +463,10 @@ QVariant db_base::data(const QModelIndex &index, int role) const
 			return item->bg_color(hd);
 		case Qt::UserRole:
 			return item->isVisible();
+		case Qt::ToolTipRole:
+			if (hd->id==HD_internal_name || item->isVisible()==1)
+				return item->column_tooltip(hd);
+			break;
 	}
 	return QVariant();
 }
@@ -537,6 +557,54 @@ void db_base::updateItem(pki_base *pki, QString name, QString comment)
 	j = index(i.row(), allHeaders.size(), i.parent());
 	emit dataChanged(i, j);
 	emit pkiChanged(pki);
+	restart_timer();
+}
+
+void db_base::timerEvent(QTimerEvent *event)
+{
+	int youngest = SECS_PER_DAY;
+	int id = event->timerId();
+
+	FOR_ALL_pki(pki, pki_base) {
+		for (int idx=0; idx < allHeaders.count(); idx++) {
+			dbheader *hd = allHeaders[idx];
+			if (hd->type != dbheader::hd_asn1time)
+				continue;
+			a1time t = pki->column_a1time(hd);
+			if (t.isUndefined())
+				continue;
+			int age = t.age();
+			if (age < 0)
+				age *= -1;
+			bool do_emit = false;
+			if (age < youngest)
+				youngest = age;
+			if (!hd->show)
+				continue;
+			if (id == secondsTimer && (age < SECS_PER_MINUTE *2 ||
+						   age % SECS_PER_MINUTE < 2))
+				do_emit = true;
+			if (id == minutesTimer && (age % SECS_PER_HOUR < 60))
+				do_emit = true;
+			if (id == hoursTimer &&
+					 (age % SECS_PER_DAY < SECS_PER_HOUR))
+				do_emit = true;
+			if (do_emit) {
+				qDebug() << "Date changed for" << pki->getIntName() << ":" << hd->getName() << "Col:" << idx << t.toSortable();
+				QModelIndex i;
+				i = createIndex(pki->row(), idx, pki);
+				emit dataChanged(i, i);
+			}
+		}
+	}
+	if (secondsTimer && youngest > SECS_PER_HOUR *2) {
+		killTimer(secondsTimer);
+		secondsTimer = 0;
+	}
+	if (minutesTimer && youngest > SECS_PER_DAY *2) {
+		killTimer(minutesTimer);
+		minutesTimer = 0;
+	}
 }
 
 void db_base::editComment(const QModelIndex &index)
@@ -649,11 +717,6 @@ void db_base::columnResetDefaults()
 	emit resetHeader();
 }
 
-bool db_base::isNumericCol(int col) const
-{
-	return isValidCol(col) ? allHeaders[col]->isNumeric() : false;
-}
-
 bool db_base::isValidCol(int col) const
 {
 	return col >= allHeaders.size() || col < 0 ? false : true;
@@ -671,3 +734,22 @@ QMimeData *db_base::mimeData(const QModelIndexList &indexes) const
 	mimeData->setData(X_XCA_DRAG_DATA, QByteArray());
 	return mimeData;
 }
+
+void db_base::writeVcalendar(const QString &fname, QStringList vcal)
+{
+	QFile file(fname);
+	file.open(QFile::ReadWrite | QFile::Truncate);
+	if (file.error()) {
+		throw errorEx(tr("Error opening file: '%1': %2")
+			.arg(fname).arg(strerror(errno)));
+		return;
+	}
+
+	QStringList ics; ics <<
+	"BEGIN:VCALENDAR" <<
+	"VERSION:2.0" <<
+	"PRODID:-//" XCA_TITLE "//" PACKAGE_VERSION "//" <<
+	vcal <<
+	"END:VCALENDAR";
+	file.write(ics.join("\r\n").toUtf8());
+}
