@@ -39,6 +39,7 @@ class db_base: public QAbstractItemModel
 	protected:
 		static QHash<quint64, pki_base*> lookup;
 		QModelIndex currentIdx;
+		int secondsTimer, minutesTimer, hoursTimer;
 		void _writePKI(pki_base *pki, bool overwrite);
 		void _removePKI(pki_base *pki );
 		void removeItem(QString k);
@@ -59,6 +60,8 @@ class db_base: public QAbstractItemModel
 		bool isValidCol(int col) const;
 		static XSqlQuery sqlSELECTpki(QString query,
 				QList<QVariant> values = QList<QVariant>());
+		void timerEvent(QTimerEvent * event);
+		void restart_timer();
 
 	public:
 		template <class T> static T *lookupPki(quint64 i)
@@ -135,7 +138,6 @@ class db_base: public QAbstractItemModel
 		void insertChild(pki_base *parent, pki_base *child);
 		void createSuccess(pki_base *pki);
 		bool columnHidden(int col) const;
-		bool isNumericCol(int col) const;
 		virtual void saveHeaderState();
 		void initHeaderView(QHeaderView *hv);
 		void setVisualIndex(int i, int visualIndex);
@@ -159,6 +161,9 @@ class db_base: public QAbstractItemModel
 		void deletePKI(QModelIndex idx);
 		QMimeData *mimeData(const QModelIndexList &indexes) const;
 		void editComment(const QModelIndex &index);
+		void emitDataChanged(pki_base *pki);
+		bool containsType(enum pki_type t) const;
+		void writeVcalendar(const QString &fname, QStringList vcal);
 
 	public slots:
 		virtual void newItem() { }
