@@ -61,18 +61,18 @@ void db_x509::dereferenceIssuer()
 dbheaderList db_x509::getHeaders()
 {
 	dbheaderList h = db_x509super::getHeaders();
-	h <<	new dbheader(HD_cert_ca,	true, tr("CA"),
+	h <<	new dbheader(HD_cert_ca, true, tr("CA"),
 			tr("reflects the basic Constraints extension")) <<
-		new dbheader(HD_cert_serial,	true, tr("Serial")) <<
-		new dbheader(HD_cert_md5fp,	false,tr("MD5 fingerprint")) <<
-		new dbheader(HD_cert_sha1fp,	false,tr("SHA1 fingerprint")) <<
-		new dbheader(HD_cert_sha256fp,	false,tr("SHA256 fingerprint")) <<
-		new dbheader(HD_cert_notBefore, false,tr("Start date"),
+		new num_dbheader(HD_cert_serial, true, tr("Serial")) <<
+		new num_dbheader(HD_cert_md5fp, false,tr("MD5 fingerprint")) <<
+		new num_dbheader(HD_cert_sha1fp,false,tr("SHA1 fingerprint")) <<
+		new num_dbheader(HD_cert_sha256fp,false,tr("SHA256 fingerprint")) <<
+		new date_dbheader(HD_cert_notBefore, false,tr("Start date"),
 				tr("Not before")) <<
-		new dbheader(HD_cert_notAfter,	true, tr("Expiry date"),
+		new date_dbheader(HD_cert_notAfter, true, tr("Expiry date"),
 				tr("Not after")) <<
-		new dbheader(HD_cert_revocation,false, tr("Revocation")) <<
-		new dbheader(HD_cert_crl_expire,true, tr("CRL Expiration"));
+		new date_dbheader(HD_cert_revocation,false, tr("Revocation")) <<
+		new date_dbheader(HD_cert_crl_expire,true, tr("CRL Expiration"));
 	return h;
 }
 
@@ -101,7 +101,6 @@ void db_x509::remFromCont(const QModelIndex &idx)
 	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
 	pki_x509 *child;
 	pki_base *new_parent;
-	QModelIndex new_idx;
 	QList<pki_x509 *> childs;
 
 	while (pki->childCount()) {
@@ -270,7 +269,7 @@ void db_x509::inToCont(pki_base *pki)
 	}
 }
 
-void db_x509::writeAllCerts(const QString fname, bool unrevoked)
+void db_x509::writeAllCerts(const QString &fname, bool unrevoked)
 {
 	bool append = false;
 	FOR_ALL_pki(pki, pki_x509) {
@@ -293,7 +292,7 @@ QList<pki_x509*> db_x509::getCerts(bool unrevoked)
 	return c;
 }
 
-void db_x509::writeIndex(const QString fname, bool hierarchy)
+void db_x509::writeIndex(const QString &fname, bool hierarchy)
 {
 	if (hierarchy) {
 		QString dir = fname + "/";
@@ -313,13 +312,18 @@ void db_x509::writeIndex(const QString fname, bool hierarchy)
 	}
 }
 
-#define SERIAL_LEN 8
 static a1int randomSerial()
 {
-	unsigned char buf[SERIAL_LEN];
-	RAND_bytes(buf, SERIAL_LEN);
+	unsigned char buf[SHA512_DIGEST_LENGTH];
+	unsigned char md[SHA512_DIGEST_LENGTH];
+
+	Entropy::seed_rng();
+
+	RAND_bytes(buf, SHA512_DIGEST_LENGTH);
+	SHA512(buf, SHA512_DIGEST_LENGTH, md);
 	a1int serial;
-	serial.setRaw(buf, SERIAL_LEN);
+	if (md[0] && md[0] < 0x80)
+		serial.setRaw(md, (int)Settings["serial_len"] / 8);
 	return serial;
 }
 
@@ -333,8 +337,12 @@ a1int db_x509::getUniqueSerial(pki_x509 *signer)
 		signer_serial = signer->getSerial();
 		revList = signer->getRevList();
 	}
-	while (true) {
+	for (int i=0; ; i++) {
+		if (i > 100)
+			throw errorEx(tr("Failed to retrieve unique random serial"));
 		serial = randomSerial();
+		if (serial == a1int(0L))
+			continue;
 		if (!signer)
 			break;
 		if (signer_serial == serial)
@@ -466,7 +474,6 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 	pki_x509req *req = NULL;
 	pki_key *signkey = NULL, *clientkey = NULL, *tempkey = NULL;
 	a1int serial;
-	x509name subject;
 	QString intname;
 
     try {
@@ -476,7 +483,6 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 		clientkey = dlg->getSelectedKey();
 		if (!clientkey)
 			return NULL;
-		subject = dlg->getX509name();
 		intname = dlg->description->text();
 	} else {
 		// A PKCS#10 Request was selected
@@ -488,10 +494,6 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 			clientkey = req->getPubKey();
 			tempkey = clientkey;
 		}
-		if (dlg->reqSubChange->isChecked())
-			subject = dlg->getX509name();
-		else
-			subject = req->getSubject();
 		intname = req->getIntName();
 	}
 	TransThrow();
@@ -501,7 +503,7 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 	// initially create cert
 	cert = new pki_x509();
 	cert->setIntName(intname);
-	cert->setSubject(subject);
+	cert->setSubject(dlg->getX509name());
 	cert->setPubKey(clientkey);
 
 	// Step 2 - select Signing
@@ -664,6 +666,13 @@ void db_x509::store(QModelIndexList list)
 			tr("PEM all")) <<
 		exportType(exportType::Index, "txt",
 			tr("Certificate Index file"));
+	if (crt->getNotAfter() > a1time())
+		types << exportType(exportType::vcalendar, "ics",
+			tr("vCalendar"));
+
+	if (crt->isCA())
+		types << exportType(exportType::vcalendar_ca, "ics",
+			tr("CA vCalendar"));
 
 	types = usual << exportType() << types;
 	ExportDialog *dlg = new ExportDialog(mainwin, tr("Certificate export"),
@@ -674,7 +683,9 @@ void db_x509::store(QModelIndexList list)
 		return;
 	}
 	QString fname = dlg->filename->text();
+	QStringList vcal;
 	QList<pki_x509*> certs;
+	QList<pki_base*> items;
 	enum exportType::etype type = dlg->type();
 	delete dlg;
 	try {
@@ -749,6 +760,20 @@ void db_x509::store(QModelIndexList list)
 					(idx.internalPointer());
 			writeIndex(fname, certs);
 			break;
+		case exportType::vcalendar:
+			foreach(QModelIndex idx, list) {
+				crt = static_cast<pki_x509*>(idx.internalPointer());
+				vcal += crt->icsVEVENT();
+			}
+			writeVcalendar(fname, vcal);
+			break;
+		case exportType::vcalendar_ca:
+			foreach(QModelIndex idx, list) {
+				crt = static_cast<pki_x509*>(idx.internalPointer());
+				vcal += crt->icsVEVENT_ca();
+			}
+			writeVcalendar(fname, vcal);
+			break;
 		default:
 			exit(1);
 		}
@@ -758,7 +783,7 @@ void db_x509::store(QModelIndexList list)
 	}
 }
 
-void db_x509::writeIndex(const QString fname, QList<pki_x509*> items)
+void db_x509::writeIndex(const QString &fname, QList<pki_x509*> items)
 {
 	QFile file(fname);
 	file.open(QFile::ReadWrite | QFile::Truncate);
