@@ -32,7 +32,10 @@ dbheaderList db_x509req::getHeaders()
 		new dbheader(HD_req_unstr_name, false, tr("Unstructured name"),
 			QString(OBJ_nid2ln(NID_pkcs9_unstructuredName))) <<
 		new dbheader(HD_req_chall_pass, false, tr("Challenge password"),
-			 QString(OBJ_nid2ln(NID_pkcs9_challengePassword)));
+			 QString(OBJ_nid2ln(NID_pkcs9_challengePassword))) <<
+		new num_dbheader(HD_req_certs, false, tr("Certificate count"),
+			 tr("Number of certificates in the database with the same public key"));
+
 	return h;
 }
 
@@ -85,7 +88,6 @@ void db_x509req::newItem(pki_temp *temp, pki_x509req *orig)
 
 		req->setIntName(dlg->description->text());
 
-		dlg->initCtx(NULL, NULL, req);
 		dlg->getReqAttributes(req);
 		req->createReq(key, xn, dlg->hashAlgo->currentHash(), dlg->getAllExt());
 		 // set the comment field
@@ -150,6 +152,21 @@ void db_x509req::signReq(QModelIndex index)
 	emit newCert(req);
 }
 
+void db_x509req::setSigned(QModelIndex index, bool signe)
+{
+	pki_x509req *req = static_cast<pki_x509req*>(index.internalPointer());
+	if (!req)
+		return;
+	req->markSigned(signe);
+	emit columnsContentChanged();
+}
+
+void db_x509req::resetX509count()
+{
+	foreach(pki_x509req *r, getAllRequests())
+		r->resetX509count();
+}
+
 QList<pki_x509req *> db_x509req::getAllRequests()
 {
 	return sqlSELECTpki<pki_x509req>("SELECT item FROM requests");
