@@ -6,18 +6,21 @@
  */
 
 
+#include <unistd.h>
 #include "func.h"
 #include "exception.h"
 #include "lib/asn1time.h"
 #include "widgets/validity.h"
 #include "widgets/XcaWarning.h"
 #include <openssl/objects.h>
+#include <openssl/sha.h>
 #include <openssl/asn1.h>
 #include <openssl/err.h>
 #include <openssl/bio.h>
 #include <openssl/buffer.h>
 
 #if defined(Q_OS_MAC)
+#include <IOKit/IOKitLib.h>
   #if QT_VERSION < 0x050000
 #include <QDesktopServices>
   #else
@@ -41,6 +44,8 @@
 #include <shlobj.h>
 #endif
 
+QString currentDB;
+
 QPixmap *loadImg(const char *name )
 {
 	return new QPixmap(QString(":") + name);
@@ -49,7 +54,7 @@ QPixmap *loadImg(const char *name )
 QStringList getLibExtensions()
 {
 	QStringList l;
-#if defined(_WIN32) || defined(USE_CYGWIN)
+#if defined(Q_OS_WIN32)
 	l << QString("*.dll") << QString("*.DLL");
 #elif defined(Q_OS_MAC)
 	l << QString("*.dylib") << QString("*.so");
@@ -59,6 +64,22 @@ QStringList getLibExtensions()
 	return l;
 }
 
+int portable_app()
+{
+	static int portable = -1;
+	if (portable == -1) {
+#if defined(Q_OS_WIN32)
+		HKEY hKey;
+		portable = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\xca", 0,
+			KEY_WOW64_32KEY|KEY_READ, &hKey) != ERROR_SUCCESS;
+#else
+		const char *p = getenv("XCA_PORTABLE");
+		portable = p && *p;
+#endif
+	}
+	return portable;
+}
+
 /* returns e.g. /usr/local/share/xca for unix systems
  * or HKEY_LOCAL_MACHINE->Software->xca for WIN32
  * (e.g. c:\Program Files\xca )
@@ -70,7 +91,6 @@ QString getPrefix()
 	static char inst_dir[100] = "";
 	char *p;
 	ULONG dwLength = 100;
-	LONG lRc;
 	HKEY hKey;
 
 	if (inst_dir[0] != '\0') {
@@ -84,19 +104,22 @@ QString getPrefix()
 		*p = '\0';
 		return QString(inst_dir);
 	}
+	if (portable_app())
+		return QString(inst_dir);
 	p = inst_dir;
 	*p = '\0';
-	lRc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\xca", 0, KEY_READ, &hKey);
-	if (lRc != ERROR_SUCCESS) {
-		XCA_WARN("Registry Key: 'HKEY_LOCAL_MACHINE->Software->xca' not found");
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\xca", 0,
+		KEY_WOW64_32KEY|KEY_READ, &hKey) != ERROR_SUCCESS)
+	{
+		XCA_WARN("Registry Key: 'HKEY_LOCAL_MACHINE\\Software\\xca' not found");
 		return QString(inst_dir);
 	}
-	lRc = RegQueryValueEx(hKey, "Install_Dir", NULL, NULL,
-			(unsigned char *)inst_dir, &dwLength);
-	if (lRc != ERROR_SUCCESS){
+	if (RegQueryValueEx(hKey, "Install_Dir", NULL, NULL,
+			(unsigned char *)inst_dir, &dwLength) != ERROR_SUCCESS)
+	{
 		XCA_WARN("Registry Key: 'HKEY_LOCAL_MACHINE->Software->xca->Install_Dir' not found");
 	}
-	lRc = RegCloseKey(hKey);
+	RegCloseKey(hKey);
 	return QString(inst_dir);
 
 #elif defined(Q_OS_MAC)
@@ -148,7 +171,7 @@ QString getLibDir()
 
 QString getDocDir()
 {
-#if defined(WIN32) || defined (Q_OS_MAC)
+#if defined(Q_OS_WIN32) || defined (Q_OS_MAC)
 	return getPrefix();
 #else
 	return QString(DOCDIR);
@@ -222,6 +245,75 @@ QString filename2QString(const char *fname)
 #endif
 }
 
+QString hostId()
+{
+	static QString id;
+	unsigned char guid[100] = "", md[SHA_DIGEST_LENGTH];
+
+	if (!id.isEmpty())
+		return id;
+
+#if defined(Q_OS_WIN32)
+#define REG_CRYPTO "SOFTWARE\\Microsoft\\Cryptography"
+#define REG_GUID "MachineGuid"
+	ULONG dwGuid = sizeof guid;
+	HKEY hKey;
+
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_CRYPTO, 0,
+			KEY_WOW64_64KEY|KEY_READ, &hKey) != ERROR_SUCCESS) {
+		XCA_WARN("Registry Key: '" REG_CRYPTO "' not found");
+	} else {
+		if (RegQueryValueEx(hKey, REG_GUID, NULL, NULL,
+			guid, &dwGuid) != ERROR_SUCCESS) {
+			XCA_WARN("Registry Key: '" REG_CRYPTO "\\" REG_GUID
+				"' not found");
+		}
+	}
+	RegCloseKey(hKey);
+
+#elif defined(Q_OS_MAC)
+	io_registry_entry_t ioRegistryRoot = IORegistryEntryFromPath(
+				kIOMasterPortDefault, "IOService:/");
+	CFStringRef uuidCf = (CFStringRef)IORegistryEntryCreateCFProperty(
+				ioRegistryRoot, CFSTR(kIOPlatformUUIDKey),
+				kCFAllocatorDefault, 0);
+
+	snprintf((char*)guid, sizeof guid, "%s", CCHAR(
+		QString::fromUtf16(CFStringGetCharactersPtr(uuidCf))
+	));
+
+	IOObjectRelease(ioRegistryRoot);
+	CFRelease(uuidCf);
+
+#else
+	QString mach_id;
+	QStringList dirs; dirs <<
+			"/etc" << "/var/lib/dbus" << "/var/db/dbus";
+	foreach(QString dir, dirs) {
+		QFile file(dir + "/machine-id");
+		if (file.open(QIODevice::ReadOnly)) {
+			QTextStream in(&file);
+			mach_id = in.readLine().trimmed();
+			file.close();
+		}
+		qDebug() << "ID:" << mach_id;
+		if (!mach_id.isEmpty()) {
+			snprintf((char*)guid, sizeof guid, "%s", CCHAR(mach_id));
+			break;
+		}
+	}
+	if (mach_id.isEmpty())
+		sprintf((char*)guid, "%ld", gethostid());
+#endif
+	guid[sizeof guid -1] = 0;
+	SHA1(guid, strlen((char*)guid), md);
+	id = QByteArray((char*)md, (int)sizeof md).toBase64().mid(0, 8);
+
+	qDebug() << "GUID:" << guid << "ID:" << id;
+
+	return id;
+}
+
 QString compressFilename(QString filename, int maxlen)
 {
 	if (filename.length() < maxlen)
@@ -418,45 +510,46 @@ QMap<int, QString> dn_translations;
 
 void dn_translations_setup()
 {
-	dn_translations[NID_countryName] = QObject::tr("Country code");
-	dn_translations[NID_stateOrProvinceName] = QObject::tr("State or Province");
-	dn_translations[NID_localityName] = QObject::tr("Locality");
-	dn_translations[NID_organizationName] = QObject::tr("Organisation");
-	dn_translations[NID_organizationalUnitName] = QObject::tr("Organisational unit");
-	dn_translations[NID_commonName] = QObject::tr("Common name");
-	dn_translations[NID_pkcs9_emailAddress] = QObject::tr("E-Mail address");
-	dn_translations[NID_serialNumber] = QObject::tr("Serial number");
-	dn_translations[NID_givenName] = QObject::tr("Given name");
-	dn_translations[NID_surname] = QObject::tr("Surname");
-	dn_translations[NID_title] = QObject::tr("Title");
-	dn_translations[NID_initials] = QObject::tr("Initials");
-	dn_translations[NID_description] = QObject::tr("Description");
-	dn_translations[NID_role] = QObject::tr("Role");
-	dn_translations[NID_pseudonym] = QObject::tr("Pseudonym");
-	dn_translations[NID_generationQualifier] = QObject::tr("Generation Qualifier");
-	dn_translations[NID_x500UniqueIdentifier] = QObject::tr("x500 Unique Identifier");
-	dn_translations[NID_name] = QObject::tr("Name");
-	dn_translations[NID_dnQualifier] = QObject::tr("DN Qualifier");
-	dn_translations[NID_pkcs9_unstructuredName] = QObject::tr("Unstructured name");
-	dn_translations[NID_pkcs9_challengePassword] = QObject::tr("Challenge password");
+#define TRANS(nid, text) dn_translations[(nid)] = QObject::tr(text)
+	TRANS(NID_countryName, "Country code");
+	TRANS(NID_stateOrProvinceName, "State or Province");
+	TRANS(NID_localityName, "Locality");
+	TRANS(NID_organizationName, "Organisation");
+	TRANS(NID_organizationalUnitName, "Organisational unit");
+	TRANS(NID_commonName, "Common name");
+	TRANS(NID_pkcs9_emailAddress, "E-Mail address");
+	TRANS(NID_serialNumber, "Serial number");
+	TRANS(NID_givenName, "Given name");
+	TRANS(NID_surname, "Surname");
+	TRANS(NID_title, "Title");
+	TRANS(NID_initials, "Initials");
+	TRANS(NID_description, "Description");
+	TRANS(NID_role, "Role");
+	TRANS(NID_pseudonym, "Pseudonym");
+	TRANS(NID_generationQualifier, "Generation Qualifier");
+	TRANS(NID_x500UniqueIdentifier, "x500 Unique Identifier");
+	TRANS(NID_name, "Name");
+	TRANS(NID_dnQualifier, "DN Qualifier");
+	TRANS(NID_pkcs9_unstructuredName, "Unstructured name");
+	TRANS(NID_pkcs9_challengePassword, "Challenge password");
 
-	dn_translations[NID_basic_constraints] = QObject::tr("Basic Constraints");
-	dn_translations[NID_subject_alt_name] = QObject::tr("Subject alternative name");
-	dn_translations[NID_issuer_alt_name] = QObject::tr("issuer alternative name");
-	dn_translations[NID_subject_key_identifier] = QObject::tr("Subject key identifier");
-	dn_translations[NID_authority_key_identifier] = QObject::tr("Authority key identifier");
-	dn_translations[NID_key_usage] = QObject::tr("Key usage");
-	dn_translations[NID_ext_key_usage] = QObject::tr("Extended key usage");
-	dn_translations[NID_crl_distribution_points] = QObject::tr("CRL distribution points");
-	dn_translations[NID_info_access] = QObject::tr("Authority information access");
-	dn_translations[NID_netscape_cert_type] = QObject::tr("Certificate type");
-	dn_translations[NID_netscape_base_url] = QObject::tr("Base URL");
-	dn_translations[NID_netscape_revocation_url] = QObject::tr("Revocation URL");
-	dn_translations[NID_netscape_ca_revocation_url] = QObject::tr("CA Revocation URL");
-	dn_translations[NID_netscape_renewal_url] = QObject::tr("Certificate renewal URL");
-	dn_translations[NID_netscape_ca_policy_url] = QObject::tr("CA policy URL");
-	dn_translations[NID_netscape_ssl_server_name] = QObject::tr("SSL server name");
-	dn_translations[NID_netscape_comment] = QObject::tr("Comment");
+	TRANS(NID_basic_constraints, "Basic Constraints");
+	TRANS(NID_subject_alt_name, "Subject alternative name");
+	TRANS(NID_issuer_alt_name, "issuer alternative name");
+	TRANS(NID_subject_key_identifier, "Subject key identifier");
+	TRANS(NID_authority_key_identifier, "Authority key identifier");
+	TRANS(NID_key_usage, "Key usage");
+	TRANS(NID_ext_key_usage, "Extended key usage");
+	TRANS(NID_crl_distribution_points, "CRL distribution points");
+	TRANS(NID_info_access, "Authority information access");
+	TRANS(NID_netscape_cert_type, "Certificate type");
+	TRANS(NID_netscape_base_url, "Base URL");
+	TRANS(NID_netscape_revocation_url, "Revocation URL");
+	TRANS(NID_netscape_ca_revocation_url, "CA Revocation URL");
+	TRANS(NID_netscape_renewal_url, "Certificate renewal URL");
+	TRANS(NID_netscape_ca_policy_url, "CA policy URL");
+	TRANS(NID_netscape_ssl_server_name, "SSL server name");
+	TRANS(NID_netscape_comment, "Comment");
 }
 
 QString appendXcaComment(QString current, QString msg)
