@@ -12,6 +12,7 @@
 #include <QString>
 #include <openssl/evp.h>
 #include <openssl/sha.h>
+#include <openssl/md5.h>
 #include <typeinfo>
 
 QRegExp pki_base::limitPattern;
@@ -35,6 +36,12 @@ QString pki_base::comboText() const
 {
 	return desc;
 }
+
+void pki_base::autoIntName()
+{
+	setIntName("");
+}
+
 void pki_base::deleteFromToken() { };
 void pki_base::deleteFromToken(slotid) { };
 void pki_base::writeDefault(const QString) { }
@@ -120,6 +127,9 @@ void pki_base::fromPEMbyteArray(QByteArray &ba, QString name)
 	BIO *bio = BIO_new_mem_buf(ba.data(), ba.length());
 	fromPEM_BIO(bio, name);
 	BIO_free(bio);
+	autoIntName();
+	if (getIntName().isEmpty())
+		setIntName(rmslashdot(name));
 }
 
 QString pki_base::rmslashdot(const QString &s)
@@ -283,8 +293,6 @@ QVariant pki_base::column_data(const dbheader *hd) const
 	switch (hd->id) {
 	case HD_internal_name:
 		return QVariant(getIntName());
-	case HD_creation:
-		return QVariant(insertion_date.toSortable());
 	case HD_comment:
 		return QVariant(comment.section('\n', 0, 0));
 	case HD_source:
@@ -292,15 +300,43 @@ QVariant pki_base::column_data(const dbheader *hd) const
 	case HD_primary_key:
 		return sqlItemId;
 	}
+	if (hd->type == dbheader::hd_asn1time) {
+		a1time t = column_a1time(hd);
+		if (!t.isUndefined())
+			return QVariant(t.toFancy());
+	}
 	return QVariant();
 }
 
+a1time pki_base::column_a1time(const dbheader *hd) const
+{
+	switch (hd->id) {
+	case HD_creation:
+		return insertion_date;
+	}
+	return a1time().setUndefined();
+}
+
 QVariant pki_base::getIcon(const dbheader *hd) const
 {
 	(void)hd;
 	return QVariant();
 }
 
+QVariant pki_base::column_tooltip(const dbheader *hd) const
+{
+	switch (hd->id) {
+	case HD_comment:
+		return QVariant(comment);
+	}
+	if (hd->type == dbheader::hd_asn1time) {
+		a1time t = column_a1time(hd);
+		if (!t.isUndefined())
+			return QVariant(t.toPretty());
+	}
+	return QVariant();
+}
+
 bool pki_base::compare(const pki_base *ref) const
 {
 	bool ret;
@@ -309,7 +345,7 @@ bool pki_base::compare(const pki_base *ref) const
 	return ret;
 }
 
-/* Signed 32 bit interger */
+/* Unsigned 32 bit integer */
 unsigned pki_base::hash(QByteArray ba)
 {
 	unsigned char md[EVP_MAX_MD_SIZE];
@@ -320,7 +356,7 @@ unsigned pki_base::hash(QByteArray ba)
 		((unsigned)md[2]<<16L) | ((unsigned)md[3]<<24L)
 		) & 0x7fffffffL;
 }
-unsigned pki_base::hash()
+unsigned pki_base::hash() const
 {
 	return hash(i2d());
 }
@@ -342,3 +378,51 @@ void pki_base::selfComment(QString msg)
 {
 	setComment(appendXcaComment(getComment(), msg));
 }
+
+static QString icsValue(QString s)
+{
+	int n = 60;
+	QStringList lines;
+
+	QString t = s.replace(QRegExp("([,;\\\\])"), "\\\\1")
+			.replace("\n", "\\n")
+			.replace("\r", "\\r");
+	qDebug() << "S:" << s;
+	for (int j = n; !s.isEmpty(); j--) {
+		QString sub = s.left(j);
+		if (sub.endsWith("\\") || sub.toUtf8().length() > n)
+			continue;
+		s.remove(0, j);
+		lines << sub;
+		j = n = 74;
+	}
+	return lines.join("\r\n ");
+}
+
+QStringList pki_base::icsVEVENT(const a1time &expires,
+	const QString &summary, const QString &description) const
+{
+	QByteArray ba = i2d();
+	unsigned char md[MD5_DIGEST_LENGTH];
+	MD5((const unsigned char *)ba.constData(), ba.length(), md);
+	QString uniqueid = formatHash(md, MD5_DIGEST_LENGTH, false);
+	QString desc = icsValue(description + "\n----------\n" + comment);
+	QString alarm = Settings["ical_expiry"];
+	return QStringList() <<
+
+	"BEGIN:VEVENT" <<
+	QString("DTSTAMP:%1").arg(a1time().toString("yyyyMMdd'T'HHmmss'Z'")) <<
+	QString("UID:EXP-%1@xca.ovh").arg(uniqueid) <<
+	"STATUS:NEEDS-ACTION" <<
+	QString("DTSTART:%1").arg(expires.toString("yyyyMMdd")) <<
+	"DURATION:P1D" <<
+	QString("SUMMARY:%1").arg(icsValue(summary)) <<
+	QString("DESCRIPTION:%1").arg(desc) <<
+	"BEGIN:VALARM" <<
+	"ACTION:EMAIL" <<
+	QString("SUMMARY:%1").arg(icsValue(summary)) <<
+	QString("DESCRIPTION:%1").arg(desc) <<
+	QString("TRIGGER:-P%1").arg(alarm) <<
+	"END:VALARM" <<
+	"END:VEVENT";
+}
