@@ -88,6 +88,7 @@ class pki_base : public QObject
 		{
 			desc = d;
 		}
+		virtual void autoIntName();
 		QString getComment() const
 		{
 			return comment;
@@ -152,6 +153,8 @@ class pki_base : public QObject
 		int row() const;
 		virtual QVariant column_data(const dbheader *hd) const;
 		virtual QVariant getIcon(const dbheader *hd) const;
+		virtual QVariant column_tooltip(const dbheader *hd) const;
+		virtual a1time column_a1time(const dbheader *hd) const;
 		virtual bool visible() const;
 		int isVisible();
 		bool childVisible() const;
@@ -169,10 +172,12 @@ class pki_base : public QObject
 		}
 		virtual void restoreSql(const QSqlRecord &rec);
 		QSqlError sqlItemNotFound(QVariant sqlId) const;
-		unsigned hash();
+		unsigned hash() const;
 		QString pki_source_name() const;
 		QString get_dump_filename(const QString &dir, QString ext);
 		void selfComment(QString msg);
+		QStringList icsVEVENT(const a1time &expires,
+		    const QString &summary, const QString &description) const;
 };
 
 Q_DECLARE_METATYPE(pki_base *);
