@@ -32,7 +32,6 @@ void pki_crl::fromPEM_BIO(BIO *bio, QString name)
 	openssl_error(name);
 	X509_CRL_free(crl);
 	crl = _crl;
-	setIntName(rmslashdot(name));
 }
 
 QString pki_crl::getMsg(msg_type msg) const
@@ -53,7 +52,7 @@ QString pki_crl::getMsg(msg_type msg) const
 	return pki_base::getMsg(msg);
 }
 
-QSqlError pki_crl::insertSqlData()
+QSqlError pki_crl::lookupIssuer()
 {
 	XSqlQuery q;
 	unsigned name_hash = getSubject().hashNum();
@@ -74,12 +73,23 @@ QSqlError pki_crl::insertSqlData()
 		}
 		verify(x);
 	}
+	return q.lastError();
+}
+
+QSqlError pki_crl::insertSqlData()
+{
+	QSqlError e = lookupIssuer();
+
+	if (e.isValid())
+		return e;
+
+	XSqlQuery q;
 	SQL_PREPARE(q, "INSERT INTO crls (item, hash, num, iss_hash, issuer, crl) "
 		  "VALUES (?, ?, ?, ?, ?, ?)");
 	q.bindValue(0, sqlItemId);
 	q.bindValue(1, hash());
 	q.bindValue(2, numRev());
-	q.bindValue(3, name_hash);
+	q.bindValue(3, (uint)getSubject().hashNum());
 	q.bindValue(4, issuer ? issuer->getSqlItemId() : QVariant());
 	q.bindValue(5, i2d_b64());
 	q.exec();
@@ -391,10 +401,6 @@ QVariant pki_crl::column_data(const dbheader *hd) const
 				return QVariant(tr("unknown"));
 		case HD_crl_revoked:
 			return QVariant(numRev());
-		case HD_crl_lastUpdate:
-			return QVariant(getLastUpdate().toSortable());
-		case HD_crl_nextUpdate:
-			return QVariant(getNextUpdate().toSortable());
 		case HD_crl_crlnumber:
 			a1int a;
 			if (getCrlNumber(&a))
@@ -404,7 +410,32 @@ QVariant pki_crl::column_data(const dbheader *hd) const
 	return pki_x509name::column_data(hd);
 }
 
+a1time pki_crl::column_a1time(const dbheader *hd) const
+{
+	switch (hd->id) {
+		case HD_crl_lastUpdate:
+			return getLastUpdate();
+		case HD_crl_nextUpdate:
+			return getNextUpdate();
+	}
+	return pki_base::column_a1time(hd);
+}
+
 QVariant pki_crl::getIcon(const dbheader *hd) const
 {
 	return hd->id == HD_internal_name ? QVariant(*icon) : QVariant();
 }
+
+QStringList pki_crl::icsVEVENT() const
+{
+	return pki_base::icsVEVENT(getNextUpdate(),
+		tr("Renew CRL: %1").arg(getIntName()),
+		tr("The XCA CRL '%1', issued by the CA '%2' on %3 will expire on %4.\n"
+		  "It is stored in the XCA database '%5'")
+			.arg(getIntName())
+			.arg(issuer ? getIssuerName() : QString())
+			.arg(getLastUpdate().toPretty())
+			.arg(getNextUpdate().toPretty())
+			.arg(currentDB)
+	);
+}
