@@ -94,6 +94,7 @@ QSqlError pki_x509::insertSqlData()
 	q.bindValue(5, (int)isCA());
 	q.bindValue(6, i2d_b64());
 	q.exec();
+	MainWindow::reqs->resetX509count();
 	if (!isCA())
 		return q.lastError();
 
@@ -171,6 +172,7 @@ QSqlError pki_x509::deleteSqlData()
 	foreach(pki_base *pki, list)
 		AffectedItems(pki->getSqlItemId());
 
+	MainWindow::reqs->resetX509count();
 	return q.lastError();
 }
 
@@ -206,16 +208,13 @@ pki_x509 *pki_x509::findIssuer()
 	return NULL;
 }
 
-void pki_x509::fromPEM_BIO(BIO *bio, QString name)
+void pki_x509::fromPEM_BIO(BIO *bio, QString)
 {
 	X509 *_cert;
 	_cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
 	pki_openssl_error();
 	X509_free(cert);
 	cert = _cert;
-	autoIntName();
-	if (getIntName().isEmpty())
-		setIntName(rmslashdot(name));
 }
 
 void pki_x509::fload(const QString fname)
@@ -738,7 +737,9 @@ bool pki_x509::verify(pki_x509 *signer)
 		return true;
 	if ((psigner != NULL) || (signer == NULL))
 		return false;
-	if (signer == this && signerSqlId == sqlItemId)
+	if (signer == this &&
+	    signerSqlId == sqlItemId &&
+	    signerSqlId != QVariant())
 		return true;
 
 	if (verify_only(signer)) {
@@ -894,7 +895,7 @@ int pki_x509::sigAlg() const
 
 pki_x509 *pki_x509::getSigner()
 {
-	return static_cast<pki_x509 *>(psigner);
+	return psigner;
 }
 
 bool pki_x509::isRevoked() const
@@ -929,18 +930,6 @@ QVariant pki_x509::column_data(const dbheader *hd) const
 	switch (hd->id) {
 		case HD_cert_serial:
 			return QVariant(getSerial().toHex());
-		case HD_cert_notBefore:
-			return QVariant(getNotBefore().toSortable());
-		case HD_cert_notAfter:
-			return QVariant(getNotAfter().toSortable());
-		case HD_cert_revocation:
-			return QVariant(isRevoked() ?
-				revocation.getDate().toSortable() : "");
-		case HD_cert_crl_expire:
-			if (canSign() && !crlExpire.isUndefined())
-				return QVariant(crlExpire.toSortable());
-			else
-				return QVariant();
 		case HD_cert_md5fp:
 			return QVariant(fingerprint(EVP_md5()));
 		case HD_cert_sha1fp:
@@ -958,17 +947,70 @@ QVariant pki_x509::column_data(const dbheader *hd) const
 				else
 					return QVariant(tr("Yes"));
 			}
-			return QVariant("");
+			break;
 		}
 	}
 	return pki_x509super::column_data(hd);
 }
 
+a1time pki_x509::column_a1time(const dbheader *hd) const
+{
+	switch (hd->id) {
+		case HD_cert_notBefore:
+			return getNotBefore();
+		case HD_cert_notAfter:
+			return getNotAfter();
+		case HD_cert_revocation:
+			if (isRevoked())
+				return revocation.getDate();
+			break;
+		case HD_cert_crl_expire:
+			if (canSign())
+				return crlExpire;
+			break;
+	}
+	return pki_base::column_a1time(hd);
+}
+
+QStringList pki_x509::icsVEVENT() const
+{
+	return pki_base::icsVEVENT(getNotAfter(),
+		tr("Renew certificate: %1").arg(getIntName()),
+		tr("The XCA certificate '%1', issued on %2 "
+		   "will expire on %3.\n"
+		   "It is stored in the XCA database '%4'")
+			.arg(getIntName())
+			.arg(getNotBefore().toPretty())
+			.arg(getNotAfter().toPretty())
+			.arg(currentDB)
+	);
+}
+
+QStringList pki_x509::icsVEVENT_ca() const
+{
+	QStringList ics;
+	ics << icsVEVENT();
+	foreach(pki_base *p, childItems) {
+		pki_x509 *pki = static_cast<pki_x509 *>(p);
+		if (pki->getNotAfter() > a1time() && !isRevoked())
+			ics << pki->icsVEVENT();
+	}
+
+	ics << pki_base::icsVEVENT(crlExpire,
+		tr("CRL Renewal of CA '%1' due").arg(getIntName()),
+		tr("The latest CRL issued by the CA '%1' will expire on %2.\n"
+		  "It is stored in the XCA database '%3'")
+			.arg(getIntName())
+			.arg(crlExpire.toPretty())
+			.arg(currentDB)
+	);
+	return ics;
+}
+
 QVariant pki_x509::getIcon(const dbheader *hd) const
 {
 	int pixnum = 0;
 	bool ca;
-	pki_key *k;
 
 	switch (hd->id) {
 	case HD_cert_ca:
@@ -979,16 +1021,13 @@ QVariant pki_x509::getIcon(const dbheader *hd) const
 		pixnum = 4;
 		break;
 	case HD_internal_name:
-		k = getRefKey();
-		if (k && k->isPrivKey()) {
+		if (hasPrivKey())
 			pixnum += 1;
-		}
-		if (isRevoked()){
+		if (isRevoked())
 			pixnum += 2;
-		}
 		break;
 	default:
-		return QVariant();
+		return pki_x509super::getIcon(hd);
 	}
 	return QVariant(*icon[pixnum]);
 }
@@ -1019,17 +1058,25 @@ QVariant pki_x509::bg_color(const dbheader *hd) const
 	if (Settings["no_expire_colors"])
 		return QVariant();
 
+	QString unit, cert_expiry_num = Settings["cert_expiry"];
+	unit = cert_expiry_num.right(1);
+	cert_expiry_num.chop(1);
+	int n = cert_expiry_num.toInt();
+
 	a1time nb, na, now, certwarn;
 
 	nb = getNotBefore();
 	na = getNotAfter();
 	now = a1time::now();
 
-	int lifetime = nb.secsTo(na);
-
-	/* warn after 4/5 certificate lifetime */
-	certwarn = na.addSecs(- lifetime /5);
-
+	if (unit == "%") {
+		quint64 lifetime = nb.secsTo(na);
+		certwarn = nb.addSecs(lifetime *n /100);
+	} else if (unit == "D") {
+		certwarn = na.addDays(-n);
+	} else if (unit == "W") {
+		certwarn = na.addDays(-n*7);
+	}
 	switch (hd->id) {
 		case HD_cert_notBefore:
 			if (nb > now || !nb.isValid() || nb.isUndefined())
