@@ -144,6 +144,7 @@ class pki_x509 : public pki_x509super
 		x509v3ext getExtByNid(int nid) const;
 		QVariant column_data(const dbheader *hd) const;
 		QVariant getIcon(const dbheader *hd) const;
+		a1time column_a1time(const dbheader *hd) const;
 		QByteArray i2d() const;
 		void d2i(QByteArray &ba);
 		void deleteFromToken();
@@ -163,6 +164,8 @@ class pki_x509 : public pki_x509super
 		QSqlError insertSqlData();
 		QSqlError deleteSqlData();
 		void restoreSql(const QSqlRecord &rec);
+		QStringList icsVEVENT() const;
+		QStringList icsVEVENT_ca() const;
 };
 
 Q_DECLARE_METATYPE(pki_x509 *);
