@@ -6,10 +6,12 @@
  */
 
 
+#include <typeinfo>
 
 #include "pki_x509.h"
 #include "pki_evp.h"
 #include "func.h"
+#include "db_base.h"
 #include "x509name.h"
 #include "exception.h"
 #include <openssl/bio.h>
@@ -28,6 +30,7 @@ pki_x509req::pki_x509req(const QString name)
 	pki_openssl_error();
 	pkiType=x509_req;
 	done = false;
+	resetX509count();
 }
 
 pki_x509req::~pki_x509req()
@@ -43,14 +46,33 @@ QSqlError pki_x509req::insertSqlData()
 	if (e.isValid())
 		return e;
 	SQL_PREPARE(q, "INSERT INTO requests (item, hash, signed, request) "
-		  "VALUES (?, ?, 0, ?)");
+		  "VALUES (?, ?, ?, ?)");
 	q.bindValue(0, sqlItemId);
 	q.bindValue(1, hash());
-	q.bindValue(2, i2d_b64());
+	q.bindValue(2, done ? 1 : 0);
+	q.bindValue(3, i2d_b64());
 	q.exec();
 	return q.lastError();
 }
 
+void pki_x509req::markSigned(bool signe)
+{
+	XSqlQuery q;
+	Transaction;
+	TransThrow();
+
+	SQL_PREPARE(q, "UPDATE requests SET signed=? WHERE item=?");
+	q.bindValue(0, signe);
+	q.bindValue(1, sqlItemId);
+	q.exec();
+
+	if (q.lastError().isValid())
+		return;
+	done = signe;
+	AffectedItems(sqlItemId);
+	TransCommit();
+}
+
 void pki_x509req::restoreSql(const QSqlRecord &rec)
 {
 	pki_x509super::restoreSql(rec);
@@ -74,8 +96,8 @@ QSqlError pki_x509req::deleteSqlData()
 
 void pki_x509req::createReq(pki_key *key, const x509name &dn, const EVP_MD *md, extList el)
 {
-	int bad_nids[] = { NID_subject_key_identifier, NID_authority_key_identifier,
-		NID_issuer_alt_name, NID_undef };
+	QList<int> bad_nids; bad_nids << NID_authority_key_identifier <<
+		NID_issuer_alt_name << NID_undef;
 
 	EVP_PKEY *privkey = NULL;
 
@@ -89,8 +111,8 @@ void pki_x509req::createReq(pki_key *key, const x509name &dn, const EVP_MD *md,
 	setSubject(dn);
 	pki_openssl_error();
 
-	for(int i=0; bad_nids[i] != NID_undef; i++)
-		el.delByNid(bad_nids[i]);
+	foreach(int nid , bad_nids)
+		el.delByNid(nid);
 
 	el.delInvalid();
 
@@ -138,9 +160,6 @@ void pki_x509req::fromPEM_BIO(BIO *bio, QString name)
 	openssl_error(name);
 	X509_REQ_free(request);
 	request = req;
-	autoIntName();
-	if (getIntName().isEmpty())
-		setIntName(rmslashdot(name));
 }
 
 void pki_x509req::fload(const QString fname)
@@ -315,6 +334,40 @@ QString pki_x509req::getAttribute(int nid) const
 	return ret.join(", ");
 }
 
+int pki_x509req::issuedCerts() const
+{
+	XSqlQuery q;
+	int count = 0;
+
+	if (x509count != -1)
+		return x509count;
+
+	SQL_PREPARE(q, "SELECT item FROM x509super WHERE key_hash=?");
+	q.bindValue(0, pubHash());
+	q.exec();
+	if (q.lastError().isValid())
+		return 0;
+	pki_key *k = getPubKey();
+	if (!k)
+		return 0;
+	while (q.next()) {
+		pki_x509super *x;
+		x = db_base::lookupPki<pki_x509super>(q.value(0));
+		if (!x) {
+			qDebug("x509 with id %d not found",
+				q.value(0).toInt());
+			continue;
+		}
+		if (typeid(*x) == typeid(pki_x509) && x->compareRefKey(k))
+			count++;
+		qDebug() << "Req:" << getIntName() << "Cert with hash"
+			 << x->getIntName() << count;
+	}
+	delete k;
+	x509count = count;
+	return count;
+}
+
 QVariant pki_x509req::column_data(const dbheader *hd) const
 {
 	switch (hd->id) {
@@ -324,6 +377,8 @@ QVariant pki_x509req::column_data(const dbheader *hd) const
 		return getAttribute(NID_pkcs9_unstructuredName);
 	case HD_req_chall_pass:
 		return getAttribute(NID_pkcs9_challengePassword);
+	case HD_req_certs:
+		return QVariant(issuedCerts());
 	}
 	return pki_x509super::column_data(hd);
 }
@@ -331,19 +386,17 @@ QVariant pki_x509req::column_data(const dbheader *hd) const
 QVariant pki_x509req::getIcon(const dbheader *hd) const
 {
 	int pixnum = -1;
-	pki_key *k;
 
 	switch (hd->id) {
 	case HD_internal_name:
-		pixnum = 0;
-		k = getRefKey();
-		if (k && k->isPrivKey())
-			pixnum = 1;
+		pixnum = hasPrivKey() ? 0 : 1;
 		break;
 	case HD_req_signed:
 		if (done)
 			pixnum = 2;
 		break;
+	default:
+		return pki_x509super::getIcon(hd);
 	}
 	if (pixnum == -1)
 		return QVariant();
