@@ -24,6 +24,7 @@ class pki_x509req : public pki_x509super
 {
 		Q_OBJECT
 
+		mutable int x509count;
 	protected:
 		X509_REQ *request;
 		bool done;
@@ -40,12 +41,14 @@ class pki_x509req : public pki_x509super
 		void fromData(const unsigned char *p, db_header_t *head);
 		x509name getSubject() const;
 		void writeReq(const QString fname, bool pem);
+		void markSigned(bool signe);
 		X509_REQ *getReq()
 		{
 			return request;
 		}
 		void addAttribute(int nid, QString content);
 		QString getAttribute(int nid) const;
+		int issuedCerts() const;
 
 		int verify() const;
 		pki_key *getPubKey() const;
@@ -58,10 +61,14 @@ class pki_x509req : public pki_x509super
 		{
 			done = d;
 		}
-		bool getDone()
+		bool getDone() const
 		{
 			return done;
 		}
+		void resetX509count() const
+		{
+			x509count = -1;
+		}
 		virtual QString getMsg(msg_type msg) const;
 		void d2i(QByteArray &ba);
 		QByteArray i2d() const;
