@@ -10,6 +10,8 @@
 #include "pki_x509super.h"
 #include "db_base.h"
 
+QPixmap *pki_x509super::icon[1];
+
 pki_x509super::pki_x509super(const QString name)
 	: pki_x509name(name)
 {
@@ -20,7 +22,7 @@ pki_x509super::~pki_x509super()
 {
 }
 
-QSqlError pki_x509super::insertSqlData()
+QSqlError pki_x509super::lookupKey()
 {
 	XSqlQuery q;
 	unsigned hash = pubHash();
@@ -43,13 +45,22 @@ QSqlError pki_x509super::insertSqlData()
 			break;
 		}
 	}
+	return q.lastError();
+}
 
+QSqlError pki_x509super::insertSqlData()
+{
+	QSqlError e = lookupKey();
+	if (e.isValid())
+		return e;
+
+	XSqlQuery q;
 	SQL_PREPARE(q, "INSERT INTO x509super (item, subj_hash, pkey, key_hash) "
 		  "VALUES (?, ?, ?, ?)");
 	q.bindValue(0, sqlItemId);
 	q.bindValue(1, (uint)getSubject().hashNum());
 	q.bindValue(2, privkey ? privkey->getSqlItemId() : QVariant());
-	q.bindValue(3, hash);
+	q.bindValue(3, pubHash());
 	q.exec();
 	return q.lastError();
 }
@@ -77,7 +88,7 @@ pki_key *pki_x509super::getRefKey() const
 	return privkey;
 }
 
-unsigned pki_x509super::pubHash()
+unsigned pki_x509super::pubHash() const
 {
 	unsigned hash = 0;
 	if (privkey) {
@@ -121,6 +132,20 @@ const EVP_MD *pki_x509super::getDigest()
 	return EVP_get_digestbynid(sigAlg());
 }
 
+bool pki_x509super::hasPrivKey() const
+{
+	pki_key *k = getRefKey();
+	return k && k->isPrivKey();
+}
+
+QVariant pki_x509super::getIcon(const dbheader *hd) const
+{
+	if (hd->id == HD_x509key_name)
+		return hasPrivKey() ? QVariant(*icon[0]) : QVariant();
+
+	return pki_base::getIcon(hd);
+}
+
 QVariant pki_x509super::column_data(const dbheader *hd) const
 {
 	if (hd->id == HD_x509key_name) {
@@ -131,7 +156,9 @@ QVariant pki_x509super::column_data(const dbheader *hd) const
 	if (hd->id == HD_x509_sigalg) {
 		return QVariant(getSigAlg());
 	}
-	if (hd->type == dbheader::hd_v3ext || hd->type == dbheader::hd_v3ext_ns) {
+	if (hd->type == dbheader::hd_v3ext ||
+	    hd->type == dbheader::hd_v3ext_ns)
+	{
 		extList el = getV3ext();
 		int idx = el.idxByNid(hd->id);
 		if (idx == -1)
