@@ -36,11 +36,12 @@ class pki_x509super : public pki_x509name
 	public:
 		pki_x509super(const QString name = "");
 		virtual ~pki_x509super();
-		unsigned pubHash();
+		unsigned pubHash() const;
 		virtual pki_key *getPubKey() const = 0;
 		virtual extList getV3ext() const = 0;
 		virtual QString getSigAlg() const;
 		virtual const EVP_MD *getDigest();
+		static QPixmap *icon[1];
 		QVariant getKeySqlId()
 		{
 			return keySqlId;
@@ -52,6 +53,9 @@ class pki_x509super : public pki_x509name
 		QVariant column_data(const dbheader *hd) const;
 		void opensslConf(QString fname);
 		bool visible() const;
+		bool hasPrivKey() const;
+		QVariant getIcon(const dbheader *hd) const;
+		QSqlError lookupKey();
 		QSqlError insertSqlData();
 		QSqlError deleteSqlData();
 		void restoreSql(const QSqlRecord &rec);
