@@ -21,7 +21,7 @@ class exportType {
 		PKCS12, PKCS12_chain, PEM_cert_key, PEM_cert_pk8,
 		PEM_key, PEM_private, PEM_private_encrypt, DER_private,
 		DER_key, PKCS8, PKCS8_encrypt, SSH2_public,
-		PEM_selected, PKCS7_selected, Index,
+		PEM_selected, PKCS7_selected, Index, vcalendar, vcalendar_ca,
 		ETYPE_max };
 	enum etype type;
 	QString desc;
