@@ -25,7 +25,7 @@
 
 QSqlError MainWindow::initSqlDB()
 {
-	QStringList schemas[5];
+	QStringList schemas[6];
 
 #include "database_schema.cpp"
 
@@ -141,7 +141,7 @@ void MainWindow::importOldDatabase(QString dbname)
 	pki_base *pki;
 	db_base *cont;
 	QList<enum pki_type> pkitype; pkitype <<
-	    smartCard << asym_key << tmpl << x509_req << x509 << revocation;
+	    smartCard << asym_key << tmpl << x509 << x509_req << revocation;
 
 	Settings["pwhash"] = pki_evp::passHash;
 	for (int i=0; i < pkitype.count(); i++) {
@@ -195,6 +195,9 @@ void MainWindow::importOldDatabase(QString dbname)
 			}
 			free(p);
 			if (pki) {
+				pki_x509req *r=dynamic_cast<pki_x509req*>(pki);
+				if (r && r->issuedCerts() > 0)
+					r->setDone();
 				qDebug() << "load old:" << pki->getIntName();
 				cont->insertPKI(pki);
 			}
@@ -258,9 +261,9 @@ int MainWindow::init_database(QString dbName)
 		return 1;
 	}
 	certView->setRootIsDecorated(db_x509::treeview);
-
+	ret = 1;
 	try {
-		if (pki_evp::passwd.isEmpty()) {
+		if (pki_evp::passwd.isEmpty() && oldDbFile.isEmpty()) {
 			ret = initPass(dbName);
 			if (ret == 2)
 				return ret;
@@ -277,6 +280,9 @@ int MainWindow::init_database(QString dbName)
 		return ret;
 	}
 
+	if (!oldDbFile.isEmpty())
+		importOldDatabase(oldDbFile);
+
 	searchEdit->setText("");
 	searchEdit->show();
 	statusBar()->addWidget(searchEdit, 1);
@@ -305,9 +311,6 @@ int MainWindow::init_database(QString dbName)
 	tempView->setModel(temps);
 	crlView->setModel(crls);
 
-	if (!oldDbFile.isEmpty())
-		importOldDatabase(oldDbFile);
-
 	set_geometry(Settings["mw_geometry"]);
 	setWindowTitle(XCA_TITLE);
 	setItemEnabled(true);
@@ -396,20 +399,22 @@ void MainWindow::undelete()
 #endif
 }
 
+static QString defaultdb()
+{
+	return getUserSettingsDir() +QDir::separator() + "defaultdb";
+}
+
 int MainWindow::open_default_db()
 {
 	if (QSqlDatabase::database().isOpen())
 		return 0;
-	FILE *fp = fopen_read(getUserSettingsDir() +
-			QDir::separator() + "defaultdb");
-	if (!fp)
+	QFile inputFile(defaultdb());
+	if (!inputFile.open(QIODevice::ReadOnly))
 		return 0;
+	QTextStream in(&inputFile);
+	QString dbfile = in.readLine();
+	inputFile.close();
 
-	char buff[256];
-	size_t len = fread(buff, 1, 255, fp);
-	fclose(fp);
-	buff[len] = 0;
-	QString dbfile = filename2QString(buff).trimmed();
 	if (QFile::exists(dbfile) || OpenDb::isRemoteDB(dbfile))
 		return init_database(dbfile);
 	return 0;
@@ -417,32 +422,28 @@ int MainWindow::open_default_db()
 
 void MainWindow::default_database()
 {
+	if (portable_app())
+		return;
+
+	QFile file(defaultdb());
 	QFileInfo fi(currentDB);
-	QString dir = getUserSettingsDir();
-	QString file = dir +QDir::separator() +"defaultdb";
-	FILE *fp;
-	QDir d;
 
 	if (currentDB.isEmpty()) {
-		QFile::remove(file);
+		file.remove();
 		return;
 	}
-	d.mkpath(dir);
 
-	fp = fopen_write(file);
-	if (fp) {
+	if (file.open(QIODevice::ReadWrite)) {
 		QByteArray ba;
 		if (OpenDb::isRemoteDB(currentDB))
 			ba = filename2bytearray(currentDB);
 		else
 			ba = filename2bytearray(fi.canonicalFilePath());
 		ba += '\n';
-		if (fwrite(ba.constData(), ba.size(), 1, fp)) {
-			/* IGNORE_RESULT */
-		}
-		fclose(fp);
+		file.write(ba);
+		/* write() failed? Harmless. Only inconvenient */
 	}
-
+	file.close();
 }
 
 void MainWindow::close_database()
@@ -511,24 +512,25 @@ void MainWindow::close_database()
 	XSqlQuery::clearTablePrefix();
 }
 
+static QString dbhistory()
+{
+	return getUserSettingsDir() + QDir::separator() + "dbhistory";
+}
+
 void MainWindow::load_history()
 {
-	QFile file;
-	QString name = getUserSettingsDir() + QDir::separator() + "dbhistory";
-
-	file.setFileName(name);
+	QString name;
+	QFile file(dbhistory());
 	if (!file.open(QIODevice::ReadOnly))
 		return;
 
 	history.clear();
 	while (!file.atEnd()) {
-		QString name;
 		char buf[1024];
 		ssize_t size = file.readLine(buf, sizeof buf);
 		if (size <= 0)
 			break;
-		name = filename2QString(buf);
-		name = name.trimmed();
+		name = filename2QString(buf).trimmed();
 		if (name.size() == 0)
 			continue;
 		if (history.indexOf(name) == -1)
@@ -548,23 +550,23 @@ void MainWindow::update_history(QString fname)
 {
 	QFile file;
 	int pos;
-	QString name, dir = getUserSettingsDir();
-	QDir d;
 
 	pos = history.indexOf(fname);
 	if (pos == 0)
 		return; /* no changes */
 
-	d.mkpath(dir);
-
 	if (pos > 0)
 		history.removeAt(pos);
 	history.prepend(fname);
 	while (history.size() > 10)
 		history.removeLast();
 
-	name = dir + QDir::separator() + "dbhistory";
-	file.setFileName(name);
+	update_history_menu();
+
+	if (portable_app())
+		return;
+
+	file.setFileName(dbhistory());
 	if (!file.open(QIODevice::ReadWrite))
 		return;
 
@@ -575,5 +577,4 @@ void MainWindow::update_history(QString fname)
 			break;
 	}
 	file.close();
-	update_history_menu();
 }
