@@ -76,7 +76,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void checkDB();
 		QSqlError initSqlDB();
 		QString openSqlDB(QString dbName);
-		QString currentDB;
+		QTimer *eachSecond;
 
 	protected:
 		void init_images();
