@@ -13,6 +13,7 @@
 #include <QMessageBox>
 #include <QLineEdit>
 #include <QLabel>
+#include <QPushButton>
 #include <QPixmap>
 #include <QPushButton>
 #include <QValidator>
@@ -28,6 +29,78 @@
 #include "lib/oid.h"
 #include "lib/func.h"
 
+void NewX509::setupExplicitDN(NIDlist my_dn_nid = NIDlist())
+{
+	NIDlist expl_dn_nid;
+
+	/* Create configured explicit_dn list */
+	if (!Settings["explicit_dn"].empty()) {
+		foreach(QString dn, Settings["explicit_dn"].split(",")) {
+			int nid = OBJ_sn2nid(CCHAR(dn));
+			if (!my_dn_nid.contains(nid))
+				expl_dn_nid << nid;
+		}
+	}
+	nameEdits = setupExplicitInputs(my_dn_nid + expl_dn_nid,
+					dnWidget, description, 2);
+}
+
+QList<nameEdit> NewX509::setupExplicitInputs(NIDlist nid_list,
+			QWidget *parent, QWidget *old, int columns)
+{
+	QList<nameEdit> edits;
+	QGridLayout *layout = dynamic_cast<QGridLayout *>(parent->layout());
+	if (layout) {
+		QLayoutItem *child;
+		while ((child = layout->takeAt(0))) {
+			delete child->widget();
+			delete child;
+		}
+	} else {
+		layout = new QGridLayout(parent);
+		layout->setAlignment(Qt::AlignTop);
+		layout->setSpacing(6);
+		layout->setMargin(0);
+	}
+	int n = 0, col = 0;
+
+	foreach(int nid, nid_list) {
+		DoubleClickLabel *label;
+		QLineEdit *edit;
+		QString trans = dn_translations[nid];
+		QString ln = OBJ_nid2ln(nid), sn = OBJ_nid2sn(nid);
+		label = new DoubleClickLabel(parent);
+		if (Settings["translate_dn"] && !trans.isEmpty()) {
+			label->setText(trans);
+			label->setToolTip(QString("[%1] %2").arg(sn, ln));
+			if (sn == ln)
+				label->setToolTip(ln);
+		} else {
+			label->setText(ln);
+			label->setToolTip(QString("[%1] %2").arg(sn, trans));
+			if (trans == sn)
+				label->setToolTip(trans);
+		}
+		label->setClickText(OBJ_nid2sn(nid));
+		connect(label, SIGNAL(doubleClicked(QString)),
+                        MainWindow::getResolver(), SLOT(searchOid(QString)));
+		edit = new QLineEdit(parent);
+		setupLineEditByNid(nid, edit);
+		edits << nameEdit(nid, edit, label);
+
+		layout->addWidget(label, n, col);
+		layout->addWidget(edit, n, col +1);
+		n++;
+		if (n > (nid_list.size()-1)/columns) {
+			col += 2;
+			n = 0;
+		}
+		QWidget::setTabOrder(old, edit);
+		old = edit;
+	}
+	return edits;
+}
+
 NewX509::NewX509(QWidget *parent)
 	:QDialog(parent)
 {
@@ -36,9 +109,6 @@ NewX509::NewX509(QWidget *parent)
 	dn_nid = *MainWindow::dn_nid;
 	aia_nid << OBJ_sn2nid("OCSP") << OBJ_sn2nid("caIssuers");
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
-	foreach(QString dn, Settings["explicit_dn"].split(","))
-		expl_dn_nid << OBJ_sn2nid(CCHAR(dn));
-
 	QStringList keys;
 
 	setupUi(this);
@@ -114,92 +184,11 @@ NewX509::NewX509(QWidget *parent)
 	X509V3_set_ctx_nodb(&ext_ctx);
 
 	// Setup dnWidget
-	if (dnWidget->layout())
-		delete dnWidget->layout();
-	QGridLayout *dnLayout = new QGridLayout(dnWidget);
-	dnLayout->setAlignment(Qt::AlignTop);
-	dnLayout->setSpacing(6);
-	dnLayout->setMargin(0);
-	int n = 1, col = 0;
-
-	description = new QLineEdit(this);
-	description->setToolTip(tr("This name is only used internally and does not appear in the resulting certificate"));
-	QLabel *label = new QLabel(this);
-	label->setText(tr("Internal name"));
-	dnLayout->addWidget(label, 0, 0);
-	dnLayout->addWidget(description, 0, 1);
-
-	QWidget::setTabOrder(description, extDNlist);
-	QWidget *old = description;
-	foreach(int nid, expl_dn_nid) {
-		DoubleClickLabel *label;
-		QLineEdit *edit;
-		QString trans = dn_translations[nid];
-
-		label = new DoubleClickLabel(this);
-		if (Settings["translate_dn"] && !trans.isEmpty()) {
-			label->setText(trans);
-			label->setToolTip(QString("[%1] %2")
-				.arg(OBJ_nid2sn(nid)).arg(OBJ_nid2ln(nid)));
-		} else {
-			label->setText(OBJ_nid2ln(nid));
-			label->setToolTip(QString("[%1] %2")
-				.arg(OBJ_nid2sn(nid)).arg(trans));
-		}
-		label->setClickText(OBJ_nid2sn(nid));
-		connect(label, SIGNAL(doubleClicked(QString)),
-                        MainWindow::getResolver(), SLOT(searchOid(QString)));
-		edit = new QLineEdit(this);
-		setupLineEditByNid(nid, edit);
-		nameEdits << nameEdit(nid, edit, label);
-
-		dnLayout->addWidget(label, n, col);
-		dnLayout->addWidget(edit, n, col +1);
-		n++;
-		if (n > expl_dn_nid.size()/2 && col == 0) {
-			col = 2;
-			n = expl_dn_nid.size() & 1 ? 0 : 1;
-		}
-		QWidget::setTabOrder(old, edit);
-		old = edit;
-	}
+	setupExplicitDN();
 
 	// Setup Request Attributes
-	if (attrWidget->layout())
-		delete attrWidget->layout();
-	QGridLayout *attrLayout = new QGridLayout(attrWidget);
-	attrLayout->setAlignment(Qt::AlignTop);
-	attrLayout->setSpacing(6);
-	attrLayout->setMargin(0);
-	old = reqSubChange;
-	n = 0;
-	foreach(int nid, attr_nid) {
-		DoubleClickLabel *label;
-		QLineEdit *edit;
-		QString trans = dn_translations[nid];
+	attrEdits = setupExplicitInputs(attr_nid, attrWidget, reqSubChange, 1);
 
-		label = new DoubleClickLabel(this);
-		if (Settings["translate_dn"] && !trans.isEmpty()) {
-			label->setText(trans);
-			label->setToolTip(QString(OBJ_nid2sn(nid)));
-		} else {
-			label->setText(QString(OBJ_nid2ln(nid)));
-			label->setToolTip(trans);
-		}
-		label->setClickText(OBJ_nid2sn(nid));
-		connect(label, SIGNAL(doubleClicked(QString)),
-                        MainWindow::getResolver(), SLOT(searchOid(QString)));
-		edit = new QLineEdit(this);
-		attrEdits << nameEdit(nid, edit, label);
-		setupLineEditByNid(nid, edit);
-
-		attrLayout->addWidget(label, n, 0);
-		attrLayout->addWidget(edit, n, 1);
-
-		QWidget::setTabOrder(old, edit);
-		old = edit;
-		n++;
-	}
 	// last polish
 	on_certList_currentIndexChanged(0);
 	certList->setDisabled(true);
@@ -797,6 +786,9 @@ x509name NewX509::getX509name(int _throw)
 	x509name x;
 	int j, row, nid;
 
+	if (fromReqCB->isChecked() && !reqSubChange->isChecked())
+		return getSelectedReq()->getSubject();
+
 	try {
 		foreach(nameEdit ne, nameEdits) {
 			x.addEntryByNid(ne.nid, ne.edit->text());
@@ -824,11 +816,19 @@ void NewX509::setX509name(const x509name &n)
 	foreach(nameEdit ne, nameEdits) {
 		ne.edit->setText("");
 	}
-	for (int i=0; i< n.entryCount(); i++) {
+
+	if (Settings["adapt_explicit_subject"]) {
+		NIDlist mydn;
+		for (int i=0; i< n.entryCount(); i++)
+			mydn << n.nid(i);
+		setupExplicitDN(mydn);
+	}
+	for (int i=0, j=0; i< n.entryCount(); i++) {
 		int nid = n.nid(i);
 		bool done = false;
 		QStringList sl = n.entryList(i);
-		foreach(nameEdit ne, nameEdits) {
+		for ( ; j < nameEdits.size(); j++) {
+			nameEdit ne(nameEdits[j]);
 			if (nid == ne.nid && ne.edit->text().isEmpty()) {
 				ne.edit->setText(sl[2]);
 				done = true;
@@ -857,15 +857,14 @@ void NewX509::setupTmpCtx()
 
 	// initially create temporary ctx cert
 	if (ctx_cert)
-		delete(ctx_cert);
+		delete ctx_cert;
 	ctx_cert = new pki_x509();
+	ctx_cert->setSubject(getX509name());
 	if (fromReqCB->isChecked()) {
 		req = getSelectedReq();
-		ctx_cert->setSubject(req->getSubject());
 		if (req)
 			key = req->getRefKey();
 	} else {
-		ctx_cert->setSubject(getX509name());
 		key = getSelectedKey();
 	}
 	if (key)
@@ -873,6 +872,7 @@ void NewX509::setupTmpCtx()
 	// Step 2 - select Signing
 	if (foreignSignRB->isChecked()) {
 		signcert = getSelectedSigner();
+		ctx_cert->setIssuer(signcert->getSubject());
 	} else {
 		signcert = ctx_cert;
 		ctx_cert->setIssuer(ctx_cert->getSubject());
@@ -1097,10 +1097,7 @@ QString NewX509::mandatoryDnRemain()
 	if (QString(Settings["mandatory_dn"]).isEmpty())
 		return QString();
 
-	if (fromReqCB->isChecked() && !reqSubChange->isChecked())
-		n = getSelectedReq()->getSubject();
-	else
-		n = getX509name();
+	n = getX509name();
 
 	for (i=0; i< n.entryCount(); i++) {
 		int j = dnl.indexOf(QString(OBJ_nid2sn(n.nid(i))));
@@ -1345,5 +1342,25 @@ void NewX509::accept()
 				break;
 		}
 	}
+	QString cn = xn.getEntryByNid(NID_commonName);
+	QStringList san = subAltName->text().split(QRegExp(" *, *"));
+	if (cn.isEmpty() && san.contains("DNS:copycn") && pt != tmpl) {
+		gotoTab(2);
+		xcaWarning msg(this, tr("The subject alternative name shall contain a copy of the common name. However, the common name is empty."));
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit extensions"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		switch (msg.exec())
+		{
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
+				return;
+			case QMessageBox::Close:
+				reject();
+				return;
+			case QMessageBox::Apply:
+				break;
+		}
+	}
 	QDialog::accept();
 }
